@@ -1,8 +1,8 @@
-#!/usr/local/bin/perl -w
+#!/usr/bin/perl -w
 use strict;
 use GRID::Machine;
 
-my $host = $ENV{GRID_REMOTE_MACHINE} || 'casiano@orion.pcg.ull.es';
+my $host = $ENV{GRID_REMOTE_MACHINE};
 
 my $machine = GRID::Machine->new( host => $host);
 
